/*
    File:               argonHexView.h
    Description:        Generates hex view text from a binary data.
                        This file is part of the ArgonLibrary package.

    Author:             Roman Komary
    created on:         21.10.2005
    last updated:       03.09.2006 (Roman Komary)

    Copyright (c) 2005-2006, Roman Komary.

    Distributed under the Boost Software License, Version 1.0. (See
    accompanying file LICENSE_1_0.ArgonLibrary.txt or copy at
    http://www.boost.org/LICENSE_1_0.txt)
*/

#ifndef __argonHexView_h
#define __argonHexView_h

#ifndef __ArgonLibrary__
#define __ArgonLibrary__            // helpful flag, if you need to make conditional compilations somewhere in your sourcecode
#endif

#ifndef _argon_no_includes
    #include "argonString.h"
#endif

namespace ArgonLibrary
{




class HexView
{
    private :
        String   m_strLineNrFmt;     // if empty string -> no line numbers included
        String   m_strItemFmt;       // the format string to use to print one item (n number of bytes, with normally n being 1)
        String   m_strItemSpacing;   // and the string to print when to indent by one item without writing an item
        String   m_strSeparator;     // the string to append after the items but before the real text characters. if empty string -> no separator
        TCHAR    m_chPlaceholder;    // this char will be used to display some char in the text char area for characters which normally do not have a displaying character.

        int      m_numItemsPerLine;
        int      m_numBytesPerItem;

        bool     m_bIncludeLineOffsetNrs;
        bool     m_bIncludeItems;
        bool     m_bIncludeTextChars;

    protected :
        PTSTR    m_pString;
        _size_t_ m_cchString;        // size of the #m_pString# buffer in characters


    public :
        HexView( int numItemsPerLine = 8 );
        ~HexView() { if( m_pString ) free( m_pString ); }

        PCTSTR getLineNrFmt() const { return m_strLineNrFmt; }
        PCTSTR getItemFmt() const { return m_strItemFmt; }
        PCTSTR getItemSpacing() const { return m_strItemSpacing; }
        PCTSTR getSeparator() const { return m_strSeparator; }
        TCHAR  getPlaceholderChar() const { return m_chPlaceholder; }

        int  getItemsPerLine() const { return m_numItemsPerLine; }
        int  getBytesPerItem() const { return m_numBytesPerItem; }
        int  getBytesPerLine() const { return m_numItemsPerLine * m_numBytesPerItem; }

        bool Settings( int numItemsPerLine,
                       int numBytesPerItem,
                       PCTSTR pstrLineNrFmt = NULL,
                       PCTSTR pstrItemFmt = NULL,
                       PCTSTR pstrItemSpacing = NULL,
                       PCTSTR pstrSeparator = NULL,
                       TCHAR chPlaceholder = (TCHAR) 0 );
        void Configure( bool bIncludeLineOffsetNrs, bool bIncludeItems, bool bIncludeTextChars )
            { m_bIncludeLineOffsetNrs = bIncludeLineOffsetNrs; m_bIncludeItems = bIncludeItems; m_bIncludeTextChars = bIncludeTextChars; }

    // Getting a line from an arbitrary Binary array.
    public :
        PCTSTR GetLine( _idx_ lineOffsetNr, const BYTE *pBuffer, _size_t_ cBuffer, PCTSTR pstrAppendNewline = NULL ) const;

    // Getting one line after each other automatically.
    public :
        bool   Restart( _idx_ startlineOffsetNr, const BYTE *pBuffer, _size_t_ cBuffer );
        PCTSTR GetNextLine( PCTSTR pstrAppendNewline = NULL );

        _idx_    getNextLineOffsetNr() const { return m_nextLineOffsetNr; }
        _size_t_ getRemainingBufferSize() const { return m_cRemainingBufferSize; }

    protected :
        _idx_       m_nextLineOffsetNr;
        const BYTE *m_pCurrBufferPtr;
        _size_t_    m_cRemainingBufferSize;

    // Getting a whole buffer in one call.
    public :
        String GetFromBuffer( _idx_ startlineOffsetNr, const BYTE *pBuffer, _size_t_ cBuffer, PCTSTR pstrAppendNewline = _T("\n") );

    protected :
        HexView( const HexView& src );
        DWORD GetFullItem( const BYTE *ptr ) const;
        DWORD GetItem( const BYTE *ptr, int cBuffer ) const;
};




}

#endif
