/*
    File:               argonSync.cpp
    Description:        Simple synchronization classes, Semaphores.
                        This file is part of the ArgonLibrary package.

    Author:             Roman Komary
    created on:         26.08.2005
    last updated:       26.10.2006 (Roman Komary)

    Copyright (c) 2005-2006, Roman Komary.

    Distributed under the Boost Software License, Version 1.0. (See
    accompanying file LICENSE_1_0.ArgonLibrary.txt or copy at
    http://www.boost.org/LICENSE_1_0.txt)
*/

#define __argonSync_cpp
#define __an_ArgonLibrary_cpp       // helpful flag, if you need to make conditional compilations e.g. in "stdafx.h"

#include "stdafx.h"
#include "argonSync.h"

namespace ArgonLibrary
{




#ifndef _WIN32_WCE

Semaphore::Semaphore( long lInitialCount, long lMaxCount, PCSTR pstrName, PSECURITY_ATTRIBUTES psaAttributes )
          :SyncObject()
{
    if( lMaxCount <= 0 ) lMaxCount = 0;
    if( lInitialCount > lMaxCount ) lInitialCount = lMaxCount;
    m_hObject = ::CreateSemaphoreA( psaAttributes, lInitialCount, lMaxCount, pstrName );
}

Semaphore::Semaphore( long lInitialCount, long lMaxCount, PCWSTR pstrName, PSECURITY_ATTRIBUTES psaAttributes )
          :SyncObject()
{
    if( lMaxCount <= 0 ) lMaxCount = 0;
    if( lInitialCount > lMaxCount ) lInitialCount = lMaxCount;
    m_hObject = ::CreateSemaphoreW( psaAttributes, lInitialCount, lMaxCount, pstrName );
}

#else

Semaphore::Semaphore( long lInitialCount, long lMaxCount, PCTSTR pstrName, PSECURITY_ATTRIBUTES psaAttributes )
          :SyncObject()
{
    if( lMaxCount <= 0 ) lMaxCount = 0;
    if( lInitialCount > lMaxCount ) lInitialCount = lMaxCount;
    m_hObject = ::CreateSemaphore( psaAttributes, lInitialCount, lMaxCount, pstrName );
}

#endif




#ifndef _WIN32_WCE

Mutex::Mutex( bool bInitiallyOwn, PCSTR pstrName, PSECURITY_ATTRIBUTES psaAttribute )
      :SyncObject()
{
    m_hObject = ::CreateMutexA( psaAttribute, bInitiallyOwn, pstrName );
}

Mutex::Mutex( bool bInitiallyOwn, PCWSTR pstrName, PSECURITY_ATTRIBUTES psaAttribute )
      :SyncObject()
{
    m_hObject = ::CreateMutexW( psaAttribute, bInitiallyOwn, pstrName );
}

#else

Mutex::Mutex( bool bInitiallyOwn, PCTSTR pstrName, PSECURITY_ATTRIBUTES psaAttribute )
      :SyncObject()
{
    m_hObject = ::CreateMutex( psaAttribute, bInitiallyOwn, pstrName );
}

#endif




#ifndef _WIN32_WCE

Ev::Ev( bool bInitialState, bool bManualReset, PCSTR pstrName, PSECURITY_ATTRIBUTES psaAttribute )
   :SyncObject()
{
    m_hObject = ::CreateEventA( psaAttribute, bManualReset, bInitialState, pstrName );
}

Ev::Ev( bool bInitialState, bool bManualReset, PCWSTR pstrName, PSECURITY_ATTRIBUTES psaAttribute )
   :SyncObject()
{
    m_hObject = ::CreateEventW( psaAttribute, bManualReset, bInitialState, pstrName );
}

#else

Ev::Ev( bool bInitialState, bool bManualReset, PCTSTR pstrName, PSECURITY_ATTRIBUTES psaAttribute )
   :SyncObject()
{
    m_hObject = ::CreateEvent( psaAttribute, bManualReset, bInitialState, pstrName );
}

#endif




singleLock::singleLock( SyncObject& object, bool bInitialLock )
{
    m_bAcquired = false;
    m_pObject = &object;
    if( bInitialLock ) Lock();
}

bool singleLock::Lock( DWORD dwTimeout )
{
    if( m_bAcquired || !m_pObject ) return true;
    m_bAcquired = m_pObject->Lock( dwTimeout );
    return m_bAcquired;
}

bool singleLock::Unlock()
{
    if( !m_pObject ) return true;
    if( m_bAcquired ) m_bAcquired = !m_pObject->Unlock();
    return !m_bAcquired;
}




}
